==> lpc4.05.09.tar.gz: lpc4/src/main.c <==
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <setjmp.h>
#include <signal.h>
#if defined(sun)
#include <alloca.h>
#endif

#include "config.h"
#include "lint.h"
#include "wiz_list.h"
#include "simulate.h"
#include "interpret.h"
#include "object.h"
#include "exec.h"
#include "lex.h"
#include "main.h"
#include "array.h"
#include "stralloc.h"

#ifdef LACIP
#include "lacip/op.h"
#endif

#ifdef SOLARIS
#include <sys/procfs.h>
#include <sys/fcntl.h>
int proc_fd;

void open_proc_fd()
{
  char proc_name[20];

  sprintf(proc_name, "/proc/%05d", getpid());
  proc_fd = open(proc_name, O_RDONLY);
  if(proc_fd < 0)
  {
    fprintf(stderr, "Couldn't open %s\n", proc_name);
  }
}
#endif

extern char *prog;
extern int current_line;

extern int lasdebug;
int d_flag = 0;	/* Run with debug */
int t_flag = 0;	/* Disable heart beat and reset */
int comp_flag = 0; /* Trace compilations */
int T_flag = 0;
int batch_mode=0; /* he he */

void init_stdin_handler();
extern int stdin_closed,stdinisatty,stdin_is_sock;

#ifdef YYDEBUG
extern int yydebug;
#endif

char *reserved_area=NULL;	/* reserved for malloc() */
struct svalue const0, const1, const_undefined;

double consts[NUM_CONSTS];

extern jmp_buf error_recovery_context;
extern int error_recovery_context_exists;

extern struct object *master_ob;
extern int eval_cost;


char hostname[100];
unsigned long signals;
void sig_lpc(int s)
{
  signal(s,sig_lpc);
  signals|=1<<s;
}

void sig_child(int s)
{
  signal(s,sig_child);
  wait(&s);
}

int main(int argc,char ** argv,char **env)
{
    extern int game_is_being_shut_down;
    extern int current_time;
    int i, new_mudlib = 0;
    char *p;

#ifdef SOLARIS
    open_proc_fd();
#endif

#ifdef MALLOC_gc
    extern void gc_init();
#endif

#ifdef MALLOC_gc
    gc_init();
#endif
    const0.type = T_NUMBER;
    const0.subtype=NUMBER_NUMBER;
    const0.u.number = 0;
    const_undefined.type = T_NUMBER;
    const_undefined.subtype=NUMBER_UNDEFINED;
    const_undefined.u.number = 0;
    const1.type = T_NUMBER;
    const1.subtype=NUMBER_NUMBER;
    const1.u.number = 1;

    gethostname(hostname,sizeof hostname);
    hostname[sizeof(hostname)-1]=0;
    
#ifdef debugmalloc
    malloc_debug(2);
#endif
    init_otable();
    init_ohash();
    init_strings();
    /*
     * Check that the definition of EXTRACT_UCHAR() is correct.
     */
    p = (char *)&i;
    *p = -10;
    if (EXTRACT_UCHAR(p) != 0x100 - 10) {
	fprintf(stderr, "Bad definition of EXTRACT_UCHAR() in config.h.\n");
	exit(1);
    }
    my_srand(get_current_time());
    current_time = get_current_time();
    for (i=0; i < NELEM(consts); i++)
	consts[i] = exp(- i / 900.0);
    init_num_args();
    reset_machine(1);

    /*
     * The flags are parsed twice !
     * The first time, we only search for the -m flag, which specifies
     * another mudlib, and the D-flags, so that they will be available
     * when compiling master.c.
     */
    for (i=1; i < argc; i++)
    {
      if (argv[i][0] != '-') continue;

      switch(argv[i][1])
      {
      case 'D':
	if (argv[i][2])
	{			/* Amylaar : allow flags to be passed down to
				   the LPC preprocessor */
	  struct lpc_predef_s *tmp;
	  
	  tmp = (struct lpc_predef_s *)
	    malloc(sizeof(struct lpc_predef_s));
	  if (!tmp) fatal("alloca failed",0,0,0,0,0,0,0,0,0);
	  tmp->flag = argv[i]+2;
	  tmp->next = lpc_predefs;
	  lpc_predefs = tmp;
	  continue;
	}
	fprintf(stderr, "Illegal flag syntax: %s\n", argv[i]);
	exit(1);

      case 'm':
	if (chdir(argv[i]+2) == -1)
	{
	  fprintf(stderr, "Bad mudlib directory: %s\n", argv[i]+2);
	  exit(1);
	}
	new_mudlib = 1;
	break;

      case 'I':
	stdin_is_sock=1;
	break;

      case 'C':
	batch_mode=1;
	i=argc; /* the rest of the flags are 'mud only' */
      }
    }

    if(!batch_mode)
    {
      if (!new_mudlib && chdir(MUD_LIB) == -1)
      {
        fprintf(stderr, "Bad mudlib directory: %s\n", MUD_LIB);
	exit(1);
      }
    }

    if (RESERVED_SIZE > 0 && !batch_mode)
	reserved_area = malloc(RESERVED_SIZE);

    if(stdin_is_sock)
    {
      stdin_closed=1;
      stdinisatty=0;
    }else{
      init_stdin_handler();
    }

    if (setjmp(error_recovery_context))
    {
      clear_state();
      fprintf(stderr,"Fatal failiure when loading master object!\n");
    }else{
      error_recovery_context_exists = 1;
      assert_master_ob_loaded();
    }

    error_recovery_context_exists = 0;
    /*
     * Make sure master_ob is never made a dangling pointer.
     * Look at apply_master_ob() for more details.
     */
    add_ref(master_ob, "main");

    if(batch_mode)
    {
      push_constant_string(BINDIR);
      push_constant_string(MUD_LIB);
      get_simul_efun(apply_master_ob("get_simul_efun", 2));
    }else{
      get_simul_efun(apply_master_ob("get_simul_efun", 0));
    }

    if(batch_mode)
    {
      push_constant_string(BINDIR);
      push_constant_string(MUD_LIB);
      set_inc_list(apply_master_ob("define_include_dirs", 2));
    }else{
      set_inc_list(apply_master_ob("define_include_dirs", 0));
    }

    init_sockets();   /* initialize efun sockets           */

    signal(SIGCHLD, sig_child);
    signal(SIGINT, sig_lpc);
    signal(SIGQUIT, sig_lpc);
    signal(SIGTERM, sig_lpc);
    signal(SIGUSR1, sig_lpc);
    signal(SIGUSR2, sig_lpc);
    signal(SIGPIPE, SIG_IGN);
#ifndef DEBUG
    if(-1!=find_function("signal",master_ob->prog))
    {
      signal(SIGSEGV, sig_lpc);
      signal(SIGFPE, sig_lpc);
      signal(SIGBUS, sig_lpc);
    }
#endif

    for (i=1; i < argc; i++)
    {
      if (argv[i][0] != '-')
      {
	fprintf(stderr, "Bad argument %s\n", argv[i]);
	exit(1);
      } else {
	/*
	 * Look at flags. -m and -o has already been tested.
	 */
	switch(argv[i][1])
	{
	case 'f':
	  eval_cost=0;
	  push_constant_string(argv[i]+2);
	  (void)apply_master_ob("flag", 1);
	  if (game_is_being_shut_down) {
	    fprintf(stderr, "Shutdown by master object.\n");
	    exit(0);
	  }
	  continue;

	case 'C':
	{
	  struct vector *fo,*env_fo;
	  int e;
	  batch_mode=1;
	  eval_cost=0;
	  push_constant_string(argv[i]+2);
	  i++;
	  fo=allocate_array(argc-i);
	  for(e=0;i<argc;i++,e++)
	  {
	    fo->item[e].u.string=make_shared_string(argv[i]);
	    fo->item[e].subtype=STRING_SHARED;
	    fo->item[e].type=T_STRING;
	  }
	  for(e=0;env[e];e++);

	  env_fo=allocate_array(e);
	  for(e=0;env[e];e++)
	  {
	    env_fo->item[e].u.string=make_shared_string(env[e]);
	    env_fo->item[e].subtype=STRING_SHARED;
	    env_fo->item[e].type=T_STRING;
	  }
	  
	  push_vector(fo);
	  push_vector(env_fo);
	  fo->ref--;
	  env_fo->ref--;
	  (void)apply_master_ob("batch", 3);
	  if (game_is_being_shut_down) {
	    exit(0);
	  }
	  i=argc; /* the rest of the flags are 'mud only' */
	}
	case 'I':
	  push_number(batch_mode);
	  (void)apply_master_ob("stdin_is_sock",1);
	  continue;

	case 'D':
	  continue;
	case 'N':
	  continue;
	case 'm':
	  continue;
	case 'l':
	  lasdebug++; continue;
	case 'd':
	  d_flag++; continue;
	case 'c':
	  comp_flag++; continue;
	case 't':
	  t_flag++; continue;
	case 'T':
	  T_flag++; continue;
#ifdef YYDEBUG
	case 'y':
	  yydebug = 1; continue;
#endif
	default:
	  fprintf(stderr, "Unknown flag: %s\n", argv[i]);
	  exit(1);
	}
      }
    }
    
    if (game_is_being_shut_down)
	exit(1);
    load_wiz_file();
#ifdef OPCPROF
    check_cost_for_instr(-1);
#endif
    backend();

    return 0;
}

char *string_copy(char *str)
{
  char *p;

  p = xalloc(strlen(str)+1);
  (void)strcpy(p, str);
  return p;
}


void debug_message_va(a,a1,a2,a3,a4,a5,a6,a7,a8,a9)
  char *a;
  void *a1,*a2,*a3,*a4,*a5,*a6,*a7,*a8,*a9;
{
    static FILE *fp = NULL;
    char deb[100];

    /* nej tack, jag k|r */
    if(batch_mode)
      return;

    if (fp == NULL)
    {
      sprintf(deb,"%s.debug.log",hostname);
      fp = fopen(deb, "w");
      if (fp == NULL)
      {
	perror(deb);
	abort();
      }
    }
    (void)fprintf(fp, a, a1,a2,a3,a4,a5,a6,a7,a8,a9);
    (void)fflush(fp);
}

void debug_message(a,a1,a2,a3,a4,a5,a6,a7,a8,a9)
  char *a;
  void *a1,*a2,*a3,*a4,*a5,*a6,*a7,*a8,*a9;
{
  debug_message_va(a,a1,a2,a3,a4,a5,a6,a7,a8,a9);
}

void debug_message_svalue(struct svalue *v)
{
  if (v == 0)
  {
    debug_message("<NULL>");
    return;
  }
  switch(v->type)
  {
  case T_NUMBER:
    debug_message("%d", v->u.number);
    return;
  case T_STRING:
    debug_message("\"%s\"", v->u.string);
    return;
  case T_OBJECT:
    debug_message("OBJ(%s)", v->u.ob->prog->name);
    return;
  case T_LVALUE:
    debug_message("Pointer to ");
    debug_message_svalue(v->u.lvalue->sval);
    return;
  default:
    debug_message("<INVALID>\n");
    return;
  }
}

#ifdef malloc
#undef malloc
#endif

int slow_shut_down_to_do = 0;

char *xalloc(int size)
{
  char *p;
  static int going_to_exit;

  if (going_to_exit) exit(3);
  if (size == 0) fatal("Tried to allocate 0 bytes.\n",0,0,0,0,0,0,0,0,0);

  if(p = malloc(size)) return p;

  do_gc(1);
  if(p = malloc(size)) return p;
  
  if (reserved_area)
  {
    free(reserved_area);
    p = "Temporary out of MEMORY. Freeing reserve.\n";
    write(1, p, strlen(p));
    reserved_area=0;
    slow_shut_down_to_do++;
    if(p = malloc(size)) return p;
  }
  going_to_exit = 1;
  p = "Totally out of MEMORY.\n";
  write(1, p, strlen(p));
  (void)dump_trace(0);
  exit(2);
}

==> lpc4.05.10.tar.gz: lpc4/src/main.c <==
#include "global.h"
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#endif
#include <math.h>
#include <setjmp.h>
#include <signal.h>
#if defined(sun)
#include <alloca.h>
#endif
#ifdef STDC_HEADERS
#include <stdarg.h>
#endif

#include "wiz_list.h"
#include "simulate.h"
#include "interpret.h"
#include "object.h"
#include "exec.h"
#include "lex.h"
#include "main.h"
#include "array.h"
#include "stralloc.h"
#include "backend.h"
#include "simul_efun.h"

#ifdef LACIP
#include "lacip/op.h"
#endif

#ifdef GETRUSAGE_THROUGH_PROCFS
#include <sys/procfs.h>
#include <sys/fcntl.h>
int proc_fd;

void open_proc_fd()
{
  char proc_name[20];

  sprintf(proc_name, "/proc/%05ld", (long)getpid());
  proc_fd = open(proc_name, O_RDONLY);
  if(proc_fd < 0)
  {
    fprintf(stderr, "Couldn't open %s\n", proc_name);
  }
}
#endif

extern char *prog;
extern int current_line;

extern int lasdebug;
int d_flag = 0;	/* Run with debug */
int t_flag = 0;	/* Disable heart beat and reset */
int comp_flag = 0; /* Trace compilations */
int T_flag = 0;
int batch_mode=0; /* he he */

void init_stdin_handler();
extern int stdin_closed,stdinisatty,stdin_is_sock;

#ifdef YYDEBUG
extern int yydebug;
#endif

char *reserved_area=NULL;	/* reserved for malloc() */
struct svalue const0, const1, const_undefined;

double consts[NUM_CONSTS];

extern jmp_buf error_recovery_context;
extern int error_recovery_context_exists;

extern struct object *master_ob;
extern int eval_cost;


char hostname[100];
unsigned long signals;
RETSIGTYPE sig_lpc(int s)
{
  signal(s,sig_lpc);
  signals|=1<<s;
}

RETSIGTYPE sig_child(int s)
{
  wait(&s);
  signal(s,sig_child);
}


int main(int argc,char ** argv,char **env)
{
  extern void init_otable(void);
  extern void init_ohash(void);
  extern void init_strings(void);
  extern void init_sockets(void);

  extern int game_is_being_shut_down;
  extern int current_time;
  int i, new_mudlib = 0;
  char *p;

#ifdef GETRUSAGE_THROUGH_PROCFS
  open_proc_fd();
#endif

#ifdef MALLOC_gc
  extern void gc_init();
#endif

#ifdef MALLOC_gc
  gc_init();
#endif
  const0.type = T_NUMBER;
  const0.subtype=NUMBER_NUMBER;
  const0.u.number = 0;
  const_undefined.type = T_NUMBER;
  const_undefined.subtype=NUMBER_UNDEFINED;
  const_undefined.u.number = 0;
  const1.type = T_NUMBER;
  const1.subtype=NUMBER_NUMBER;
  const1.u.number = 1;

  gethostname(hostname,sizeof hostname);
  hostname[sizeof(hostname)-1]=0;
    
#ifdef debugmalloc
  malloc_debug(2);
#endif
  init_otable();
  init_ohash();
  init_strings();
  /*
   * Check that the definition of EXTRACT_UCHAR() is correct.
   */
  p = (char *)&i;
  *p = -10;
  if (EXTRACT_UCHAR(p) != 0x100 - 10) {
    fprintf(stderr, "Bad definition of EXTRACT_UCHAR() in machine.h.\n");
    exit(1);
  }
  my_srand(get_current_time());
  current_time = get_current_time();
  for (i=0; i < NELEM(consts); i++)
    consts[i] = exp(- i / 900.0);
  init_num_args();
  reset_machine(1);

  /*
   * The flags are parsed twice !
   * The first time, we only search for the -m flag, which specifies
   * another mudlib, and the D-flags, so that they will be available
   * when compiling master.c.
   */
  for (i=1; i < argc; i++)
  {
    if (argv[i][0] != '-') continue;

    switch(argv[i][1])
    {
    case 'D':
      if (argv[i][2])
      {				/* Amylaar : allow flags to be passed down to
				   the LPC preprocessor */
	  struct lpc_predef_s *tmp;
	  
	  tmp = (struct lpc_predef_s *)
	    malloc(sizeof(struct lpc_predef_s));
	  if (!tmp) fatal("alloca failed");
	  tmp->flag = argv[i]+2;
	  tmp->next = lpc_predefs;
	  lpc_predefs = tmp;
	  continue;
	}
	fprintf(stderr, "Illegal flag syntax: %s\n", argv[i]);
	exit(1);

      case 'm':
	if (chdir(argv[i]+2) == -1)
	{
	  fprintf(stderr, "Bad mudlib directory: %s\n", argv[i]+2);
	  exit(1);
	}
	new_mudlib = 1;
	break;

      case 'I':
	stdin_is_sock=1;
	break;

      case 'C':
	batch_mode=1;
	i=argc; /* the rest of the flags are 'mud only' */
      }
    }

    if(!batch_mode)
    {
      if (!new_mudlib && chdir(MUD_LIB) == -1)
      {
        fprintf(stderr, "Bad mudlib directory: %s\n", MUD_LIB);
	exit(1);
      }
    }

    if (RESERVED_SIZE > 0 && !batch_mode)
	reserved_area = malloc(RESERVED_SIZE);

    if(stdin_is_sock)
    {
      stdin_closed=1;
      stdinisatty=0;
    }else{
      init_stdin_handler();
    }

    if (setjmp(error_recovery_context))
    {
      clear_state();
      fprintf(stderr,"Fatal failiure when loading master object!\n");
    }else{
      error_recovery_context_exists = 1;
      assert_master_ob_loaded();
    }

    error_recovery_context_exists = 0;
    /*
     * Make sure master_ob is never made a dangling pointer.
     * Look at apply_master_ob() for more details.
     */
    add_ref(master_ob, "main");

    if(batch_mode)
    {
      push_constant_string(BINDIR);
      push_constant_string(MUD_LIB);
      get_simul_efun(apply_master_ob("get_simul_efun", 2));
    }else{
      get_simul_efun(apply_master_ob("get_simul_efun", 0));
    }

    if(batch_mode)
    {
      push_constant_string(BINDIR);
      push_constant_string(MUD_LIB);
      set_inc_list(apply_master_ob("define_include_dirs", 2));
    }else{
      set_inc_list(apply_master_ob("define_include_dirs", 0));
    }

    init_sockets();   /* initialize efun sockets           */

    signal(SIGCHLD, sig_child);
    signal(SIGINT, sig_lpc);
    signal(SIGQUIT, sig_lpc);
    signal(SIGTERM, sig_lpc);
    signal(SIGUSR1, sig_lpc);
    signal(SIGUSR2, sig_lpc);
    signal(SIGPIPE, SIG_IGN);
#ifndef DEBUG
    if(-1!=find_function("signal",master_ob->prog))
    {
      signal(SIGSEGV, sig_lpc);
      signal(SIGFPE, sig_lpc);
      signal(SIGBUS, sig_lpc);
    }
#endif

    for (i=1; i < argc; i++)
    {
      if (argv[i][0] != '-')
      {
	fprintf(stderr, "Bad argument %s\n", argv[i]);
	exit(1);
      } else {
	/*
	 * Look at flags. -m and -o has already been tested.
	 */
	switch(argv[i][1])
	{
	case 'f':
	  eval_cost=0;
	  push_constant_string(argv[i]+2);
	  (void)apply_master_ob("flag", 1);
	  if (game_is_being_shut_down) {
	    fprintf(stderr, "Shutdown by master object.\n");
	    exit(0);
	  }
	  continue;

	case 'C':
	{
	  struct vector *fo,*env_fo;
	  int e;
	  batch_mode=1;
	  eval_cost=0;
	  push_constant_string(argv[i]+2);
	  i++;
	  fo=allocate_array(argc-i);
	  for(e=0;i<argc;i++,e++)
	  {
	    fo->item[e].u.string=make_shared_string(argv[i]);
	    fo->item[e].subtype=STRING_SHARED;
	    fo->item[e].type=T_STRING;
	  }
	  for(e=0;env[e];e++);

	  env_fo=allocate_array(e);
	  for(e=0;env[e];e++)
	  {
	    env_fo->item[e].u.string=make_shared_string(env[e]);
	    env_fo->item[e].subtype=STRING_SHARED;
	    env_fo->item[e].type=T_STRING;
	  }
	  
	  push_vector(fo);
	  push_vector(env_fo);
	  fo->ref--;
	  env_fo->ref--;
	  (void)apply_master_ob("batch", 3);
	  if (game_is_being_shut_down) {
	    exit(0);
	  }
	  i=argc; /* the rest of the flags are 'mud only' */
	}
	case 'I':
	  push_number(batch_mode);
	  (void)apply_master_ob("stdin_is_sock",1);
	  continue;

	case 'D':
	  continue;
	case 'N':
	  continue;
	case 'm':
	  continue;
	case 'l':
	  lasdebug++; continue;
	case 'd':
	  d_flag++; continue;
	case 'c':
	  comp_flag++; continue;
	case 't':
	  t_flag++; continue;
	case 'T':
	  T_flag++; continue;
#ifdef YYDEBUG
	case 'y':
	  yydebug = 1; continue;
#endif
	default:
	  fprintf(stderr, "Unknown flag: %s\n", argv[i]);
	  exit(1);
	}
      }
    }
    
    if (game_is_being_shut_down)
	exit(1);
    load_wiz_file();
#ifdef OPCPROF
    check_cost_for_instr(-1);
#endif
    backend();

    return 0;
}

char *string_copy(char *str)
{
  char *p;

  p = xalloc(strlen(str)+1);
  (void)strcpy(p, str);
  return p;
}


void debug_message_va(char *a,va_list args)
{
  static FILE *fp = NULL;
  char deb[100];

  /* nej tack, jag k|r */
  if(batch_mode)
    return;

  if (fp == NULL)
  {
    sprintf(deb,"%s.debug.log",hostname);
    fp = fopen(deb, "w");
    if (fp == NULL)
    {
      perror(deb);
      abort();
    }
  }
  (void)VFPRINTF(fp, a, args);
  (void)fflush(fp);
}

void debug_message(char *a, ...)
{
  va_list args;
  va_start(args,a);
  debug_message_va(a,args);
  va_end(args);
}

void debug_message_svalue(struct svalue *v)
{
  if (v == 0)
  {
    debug_message("<NULL>");
    return;
  }
  switch(v->type)
  {
  case T_NUMBER:
    debug_message("%d", v->u.number);
    return;
  case T_STRING:
    debug_message("\"%s\"", v->u.string);
    return;
  case T_OBJECT:
    debug_message("OBJ(%s)", v->u.ob->prog->name);
    return;
  case T_LVALUE:
    debug_message("Pointer to ");
    debug_message_svalue(v->u.lvalue->sval);
    return;
  default:
    debug_message("<INVALID>\n");
    return;
  }
}

#ifdef malloc
#undef malloc
#endif

int slow_shut_down_to_do = 0;

char *xalloc(int size)
{
  extern void do_gc(int);
  char *p;
  static int going_to_exit;

  if (going_to_exit) exit(3);
  if (size == 0) fatal("Tried to allocate 0 bytes.\n");

  if((p = malloc(size))) return p;

  do_gc(1);
  if((p = malloc(size))) return p;
  
  if (reserved_area)
  {
    free(reserved_area);
    p = "Temporary out of MEMORY. Freeing reserve.\n";
    write(1, p, strlen(p));
    reserved_area=0;
    slow_shut_down_to_do++;
    if((p = malloc(size))) return p;
  }
  going_to_exit = 1;
  p = "Totally out of MEMORY.\n";
  write(1, p, strlen(p));
  (void)dump_trace(0);
  exit(2);
}
